package b4a.game.helper;


import java.util.ArrayList;
import java.util.List;

import com.google.android.gms.games.GamesClient;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.OnInvitationReceivedListener;
import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMessage;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMessageReceivedListener;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.realtime.RoomConfig;
import com.google.android.gms.games.multiplayer.realtime.RoomStatusUpdateListener;
import com.google.android.gms.games.multiplayer.realtime.RoomUpdateListener;

import b4a.game.helper.GameHelper.GameHelperListener;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.preference.PreferenceManager.OnActivityResultListener;
import android.util.Log;
import android.view.WindowManager;
import android.widget.Toast;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.IOnActivityResult;

@Permissions(values = {}) 

@ShortName("gGoogleGameService")
@Version(value = .5f)
@ActivityObject
@DependsOn(values = { "googleplayservices", })
@Events(values = {"OnSignInSucceeded",
		"OnSignOutComplete",
		"OnDisconnected",
		"OnSignInFailed"})

public class GameService implements GameHelper.GameHelperListener {
//, RealTimeMessageReceivedListener, RoomStatusUpdateListener, RoomUpdateListener, OnInvitationReceivedListener{
	
	protected Activity mActivity;
	private BA mBA;
	private GameClientWrapper myGCW;
    // The game helper object. This class is mainly a wrapper around this object.
    protected GameHelper mHelper;
 // request codes we use when invoking an external activity
    final int RC_RESOLVE = 5000, RC_UNUSED = 5001;

    // We expose these constants here because we don't want users of this class
    // to have to know about GameHelper at all.
    public static final int CLIENT_GAMES = GameHelper.CLIENT_GAMES;
    public static final int CLIENT_APPSTATE = GameHelper.CLIENT_APPSTATE;
    public static final int CLIENT_PLUS = GameHelper.CLIENT_PLUS;
    public static final int CLIENT_ALL = GameHelper.CLIENT_ALL;

 // Request codes for the UIs that we show with startActivityForResult:
    final static int RC_SELECT_PLAYERS = 10000;
    final static int RC_INVITATION_INBOX = 10001;
    final static int RC_WAITING_ROOM = 10002;
    
    // Requested clients. By default, that's just the games client.
    protected int mRequestedClients = CLIENT_GAMES;
	private String eventName;
	private IOnActivityResult ion;
	public GameClientWrapper gcw;

	 // Room ID where the currently active game is taking place; null if we're
    // not playing.
    String mRoomId = null;

    // Are we playing in multiplayer mode?
    boolean mMultiplayer = false;

    // The participants in the currently active game
    ArrayList<Participant> mParticipants = null;

    // My participant ID in the currently active game
    String mMyId = null;

    // If non-null, this is the id of the invitation we received via the
    // invitation listener
    String mIncomingInvitationId = null;

    // Message buffer for sending messages
    byte[] mMsgBuf = new byte[2];

    // flag indicating whether we're dismissing the waiting room because the
    // game is starting
    boolean mWaitRoomDismissedFromCode = false;
	
	
    /**
     * Constructs a BaseGameActivity with the requested clients.
     * @param requestedClients The requested clients (a combination of CLIENT_GAMES,
     *         CLIENT_PLUS and CLIENT_APPSTATE).
     */
    @Hide
    public void BaseGameServiceWithClients(int requestedClients) {

        setRequestedClients(requestedClients);
      
    }

    /**
     * Sets the requested clients. The preferred way to set the requested clients is
     * via the constructor, but this method is available if for some reason your code
     * cannot do this in the constructor. This must be called before onCreate in order to
     * have any effect. If called after onCreate, this method is a no-op.
     *
     * @param requestedClients A combination of the flags CLIENT_GAMES, CLIENT_PLUS
     *         and CLIENT_APPSTATE, or CLIENT_ALL to request all available clients.
     */
    protected void setRequestedClients(int requestedClients) {
        mRequestedClients = requestedClients;
    }
	
   
	@Override
	@Hide
	public void onSignInFailed() {
    	mBA.raiseEvent(this, eventName + "_onsigninfailed");
	}

   
    @Hide
	@Override
	public void onSignInSucceeded() {
    	mBA.raiseEvent(this , eventName + "_onsigninsucceeded");
	}
    
   @Hide
    public void onSignOutComplete() {
    	mBA.raiseEvent(this, eventName + "_onsignoutcomplete");
    }
	
	public void initialize(BA ba, String EventName){
		mActivity = ba.activity;
		mHelper = new GameHelper(ba, mActivity);	
		mBA = ba;
		eventName = EventName.toLowerCase(BA.cul);
		mHelper.setup(this, mRequestedClients);
	}
	
	/**
	 * Call this after intialization and setting up required clients(if more than just the default Games client) 
	 */
	public void beginUserInitiatedSignIn() {
        mHelper.beginUserInitiatedSignIn();
    }
	
	/**
	 * Call this to sign out at user request
	 */
	public void signOut() {
		mHelper.signOut();
	}
	
	/**
	 * Returns True if connected to Google Services
	 */
	public boolean isSignedIn() {
		return mHelper.isSignedIn();
	}
	
	/**
	 * required in Activity Resume
	 */
	public void onStart(final BA ba, boolean autoConnect) {
			
			if (mHelper.getGamesClient().isConnected()){
				//Toast.makeText(mBA.context, "on Start and connected", Toast.LENGTH_LONG).show();
				return;
			}
			if (mHelper.getGamesClient().isConnecting()) {
				//Toast.makeText(mBA.context, "on Start and connecting", Toast.LENGTH_LONG).show();
				return;
			}
			mHelper.onStart(ba, ba.activity, autoConnect);
	}
	
	
	/**
	 * Shows Achievements in a new window
	 */
	public void showAchievements(final BA ba) {
		
	    if (!mHelper.isSignedIn()) {
	    	return;
	    }
	    Intent i = new Intent(mHelper.getGamesClient().getAchievementsIntent());
	    
	    	//Toast.makeText(mBA.context, "past null", Toast.LENGTH_SHORT).show();
	  
	    ion = new IOnActivityResult() {
	        @Override
	        public void ResultArrived(int resultCode, Intent intent) {
	           //add code here if desired
	        }
	    };
	    
	    ba.startActivityForResult(ion, i);
	    
	}
	
	/**
	 * Will show the request leaderboard in a new activity
	 */
	public void showLeaderboards(final BA ba, String leaderboardId){
		if(!mHelper.isSignedIn()) {
			return;
		}
		Intent i = new Intent(mHelper.getGamesClient().getLeaderboardIntent(leaderboardId));
		
		ion = new IOnActivityResult() {
	        @Override
	        public void ResultArrived(int resultCode, Intent intent) {
	            if (resultCode == Activity.RESULT_OK) {
	        
	            }
	        }
	    };
	    ba.startActivityForResult(ion, i);
		
	}
	
	public void onStop(boolean UserClosed) {
		mHelper.onStop(UserClosed);
		//Toast.makeText(mBA.context, "on Stop", Toast.LENGTH_LONG).show();
	}
	
	/**
	 * returns null if no current GameClient exists
	 */
	public  GameClientWrapper getGamesClient() {
		//GameClientWrapper gw = new GameClientWrapper();
		if (!mHelper.isSignedIn()){
			myGCW = null;
			return null;
		}
		
		if (myGCW == null){
			myGCW = new GameClientWrapper(mHelper, mBA, eventName);
			return myGCW;
		} else if(myGCW.me == mHelper.getGamesClient()) {
			return myGCW;
		}
		myGCW = new GameClientWrapper(mHelper, mBA, eventName);
		return myGCW;
    }
	
//	public String getInvitationId() {
//        return mHelper.getInvitationId();
//    }
	
//    // Accept the given invitation.
//    void acceptInviteToRoom(String invId) {
//        // accept the invitation
//     //   Log.d(TAG, "Accepting invitation: " + invId);
//        RoomConfig.Builder roomConfigBuilder = RoomConfig.builder(this);
//        roomConfigBuilder.setInvitationIdToAccept(invId)
//                .setMessageReceivedListener(this)
//                .setRoomStatusUpdateListener(this);
////        switchToScreen(R.id.screen_wait);
////        keepScreenOn();
////        resetGameVars();
//        getGamesClient().joinRoom(roomConfigBuilder.build());
//    }
	
//    @Hide
//    @Override
//    public void onJoinedRoom(int statusCode, Room room) {
//
//        if (statusCode != GamesClient.STATUS_OK) {
//        	//need to return result to B4A
//        	mBA.raiseEvent(this, eventName + "_onjoinedroom", room);
//            return;
//        }
//
//        // show the waiting room UI
//       // showWaitingRoom(room);
//    }

    // Show the waiting room UI to track the progress of other players as they enter the
    // room and get connected.
	@Hide
    public void showWaitingRoom(final BA ba, Room room) {
        mWaitRoomDismissedFromCode = false;

        // minimum number of players required for our game
        final int MIN_PLAYERS = 2;
        
        if (!mHelper.isSignedIn()) {
	    	return;
	    }
        Intent i = getGamesClient().getRealTimeWaitingRoomIntent(room, MIN_PLAYERS);
	    
	    	//Toast.makeText(mBA.context, "past null", Toast.LENGTH_SHORT).show();
	  
	    ion = new IOnActivityResult() {
	        @Override
	        public void ResultArrived(int resultCode, Intent intent) {
	            if (resultCode == Activity.RESULT_OK) {
	            	
	            }
	        }
	    };
	 // show waiting room UI
	    ba.startActivityForResult(ion, i);
    }
    
	/**
	 * 
	 * Set the messages to display when signing in or out of Google
	 * (this is a requirement per Google to use this library)
	 */
	public void setSignInMessages(String signingInMessage, String signingOutMessage) {
        mHelper.setSigningInMessage(signingInMessage);
        mHelper.setSigningOutMessage(signingOutMessage);
    }

	@Hide
	//Don't think I need this
	public boolean onActivityResult(int arg0, int arg1, Intent arg2) {
		Toast.makeText(mBA.context, "onActivityResult", Toast.LENGTH_LONG).show();
		mHelper.onActivityResult(arg0, arg1, arg2);
		return false;
	}
	
	@Hide
	@Override
	public void onDisconnected() {
		mBA.raiseEvent(this, eventName + "_ondisconnected");
	}


}
