package b4a.game.helper;

import android.database.CharArrayBuffer;
import android.net.Uri;
import android.widget.ImageView;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;

import com.google.android.gms.common.images.ImageManager;
import com.google.android.gms.games.leaderboard.LeaderboardScore;

@ShortName(value = "gLeaderboardScore")

public class LeaderboardScoreWrapper {
	LeaderboardScore me;
	
	@Hide
	public LeaderboardScoreWrapper(LeaderboardScore ls){
		me = ls;
	}
	

	/**
	 * Retrieves a formatted string to display for this rank.
	 */

	public String getDisplayRank() {
		return me.getDisplayRank();
	}



	/**
	 * Retrieves a formatted string to display for this score.
	 */
	public String getDisplayScore() {
		return me.getDisplayScore();
	}

	/**
	 * Retrieves the rank returned from the server for this score.
	 */
	public long getRank() {
		return me.getRank();
	}

	/**
	 * Retrieves the raw score value.
	 */
	public long getRawScore() {
		return me.getRawScore();
	}

	
	/**
	 * Retrieves the gPlayer that scored this particular score
	 */
	public PlayerWrapper getScoreHolder(){
		return new PlayerWrapper(me.getScoreHolder());
	}

	/**
	 * Retrieves the name to display for the player who scored this score
	 */

	public String getScoreHolderDisplayName() {
		return me.getScoreHolderDisplayName();
	}

	@Hide

	public void getScoreHolderDisplayName(CharArrayBuffer arg0) {
		me.getScoreHolderDisplayName(arg0);
		
	}

	
	/**
	 * Pass ImageView to asynchrously load the image, if no image, nothing loaded
	 */
	public void loadScoreHolderIconImageUri(final BA ba, ImageViewWrapper ivw) {
			loadUriIntoImageview(me.getScoreHolderIconImageUri(), ivw.getObject(), ba);
	}
	
	/**
	 * Pass ImageView to asynchrously load the image, if no image, nothing loaded
	 */
	public void loadScoreHolderHiResImageUri(final BA ba, ImageViewWrapper ivw) {
			loadUriIntoImageview(me.getScoreHolderHiResImageUri(), ivw.getObject(), ba);
	}
	
	//for use by all the load uri methods
	private void loadUriIntoImageview(Uri uri, ImageView iv, BA ba){
		if (uri == null) { return;}
		ImageManager im = ImageManager.create(ba.context);
		im.loadImage(iv, uri);
	}

	/**
	 * Retrieves the timestamp (in milliseconds from epoch) at which this score was achieved.
	 */
	public long getTimestampMillis() {
		return me.getTimestampMillis();
	}



}
