package b4a.game.helper;

import java.util.ArrayList;

import com.google.android.gms.common.images.ImageManager;
import com.google.android.gms.games.leaderboard.Leaderboard;
import com.google.android.gms.games.leaderboard.LeaderboardVariant;

import android.database.CharArrayBuffer;
import android.net.Uri;
import android.widget.ImageView;
import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.objects.ImageViewWrapper;
import anywheresoftware.b4a.objects.collections.List;

@ShortName(value = "gLeaderboard")
public class LeaderboardWrapper extends AbsObjectWrapper<Leaderboard>{
	@Hide
	public void innerInitialize(final BA ba, final String eventName, boolean keepOldObject) {
		if (!keepOldObject) {
			setObject(new Leaderboard() {
				
				@Override
				public ArrayList<LeaderboardVariant> getVariants() {
					return getObject().getVariants();
				}
				
				@Override
				public int getScoreOrder() {
					return getObject().getScoreOrder();
				}
				
				@Override
				public String getLeaderboardId() {
					return getObject().getLeaderboardId();
				}
				
				@Override
				public Uri getIconImageUri() {

					return getObject().getIconImageUri();
				}
				
				@Override
				@Hide
				public void getDisplayName(CharArrayBuffer arg0) {
					getObject().getDisplayName(arg0);
					
				}
				
				@Override
				public String getDisplayName() {
					return getObject().getDisplayName();
				}
			}); 
		}
	}
	
	/**
	 * Returns a List of gLeaderboardVariant
	 */
	public List getVariants() {
		List list1 = new List();
		list1.Initialize();
		for (LeaderboardVariant lv : getObject().getVariants() ) {
			LeaderboardVariantWrapper lvw = new LeaderboardVariantWrapper(lv);
			list1.Add(lvw);
		}
		return list1;
	}
	
	/**
	 * Retrieves the sort order of scores for this leaderboard
	 */
	public int getScoreOrder() {
		return getObject().getScoreOrder();
	}
	
	/**
	 * Retrieves the ID of this leaderboard
	 */
	public String getLeaderboardId() {
		return getObject().getLeaderboardId();
	}
	
	@Hide
	public Uri getIconImageUri() {

		return getObject().getIconImageUri();
	}
	
	
	/**
	 * Pass ImageView to asynchrously load the image, if no image, nothing loaded
	 */
	public void loadIconImage(final BA ba, ImageViewWrapper ivw) {
			loadUriIntoImageview(getObject().getIconImageUri(), ivw.getObject(), ba);
	}
	
	//for use by all the load uri methods
	private void loadUriIntoImageview(Uri uri, ImageView iv, BA ba){
		if (uri == null) { return;}
		ImageManager im = ImageManager.create(ba.context);
		im.loadImage(iv, uri);
	}
	
	@Hide
	public void getDisplayName(CharArrayBuffer arg0) {
		getObject().getDisplayName(arg0);
		
	}
	
	/**
	 * Retrieves the display name of this leaderboard.
	 */
	public String getDisplayName() {
		return getObject().getDisplayName();
	}
	
	/**
	 * Score order constant for leaderboards where scores are sorted in descending order
	 */
	public int SCORE_ORDER_LARGER_IS_BETTER(){
		return getObject().SCORE_ORDER_LARGER_IS_BETTER;
	}
	
	/**
	 * Score order constant for leaderboards where scores are sorted in ascending order
	 */
	public int SCORE_ORDER_SMALLER_IS_BETTER(){
		return getObject().SCORE_ORDER_SMALLER_IS_BETTER;
	}
	
	
}
