package b4a.game.helper;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.IOnActivityResult;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;




import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.games.GamesActivityResultCodes;
import com.google.android.gms.games.GamesClient;
import com.google.android.gms.games.OnPlayersLoadedListener;
import com.google.android.gms.games.Player;
import com.google.android.gms.games.PlayerBuffer;
import com.google.android.gms.games.multiplayer.Invitation;
import com.google.android.gms.games.multiplayer.InvitationBuffer;
import com.google.android.gms.games.multiplayer.OnInvitationReceivedListener;
import com.google.android.gms.games.multiplayer.OnInvitationsLoadedListener;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMessage;
import com.google.android.gms.games.multiplayer.realtime.RealTimeMessageReceivedListener;
import com.google.android.gms.games.multiplayer.realtime.RealTimeReliableMessageSentListener;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.realtime.RoomConfig;
import com.google.android.gms.games.multiplayer.realtime.RoomStatusUpdateListener;
import com.google.android.gms.games.multiplayer.realtime.RoomUpdateListener;
//import anywheresoftware.b4a.objects.collections.List;
@ActivityObject
@ShortName(value = "gRoomConfig")
@Events(values = { "OnRoomCreated(statusCode as int, room as Object)",
			"OnInvitePlayersComplete ((listPlayerIds As List))",
			"OnWaitingRoomResponse(resultCode as int, intent as Object)",
			"OnInvitationsComplete (resultCode as int, invitation as gInvitation)",
			"OnPeerDeclined (aRoom as gRoom, listParticipantIDs as List)",
			"OnPeerInvitedToRoom (aRoom as gRoom, listParticipantIDs as List)",
			"OnPeerJoined (aRoom as gRoom, listParticipantIDs as List)",
			"OnPeerLeft (aRoom as gRoom, listParticipantIDs as List)",
			"OnPeersConnected (aRoom as gRoom, listParticipantIDs as List)",
			"OnPeersDisconnected (aRoom as gRoom, listParticipantIDs as List)",
			"OnRoomAutoMatching (aRoom as gRoom)",
			"OnRoomConnecting (aRoom as gRoom)", 
			"OnJoinedRoom (statusCode as int, aRoom as gRoom)",
			"OnRoomConnected (statusCode as int, aRoom as gRoom)",
			"OnInvitationsLoaded (listGInvitations as List)",
			"OnPlayersLoaded (listGPlayers as List)",
			"OnConnectedToRoom (aRoom as gRoom)",
			"OnInvitationRecieved (invite as gInvitation)",
			"OnLeftRoom (statusCode as int, roomID as String)",
			"OnRealTimeMessageRecieved (isReliable as Boolean, buffer() as byte, participantID as String)",
			"OnRealTimeMessageSent (statusCode as int, tokenID as int, participantID as String)"})
/**
 * Used to develop required RoomConfig for use in creating a Room object
 *
 */
public class RoomConfigWrapper implements RoomUpdateListener, RoomStatusUpdateListener,
RealTimeMessageReceivedListener, RealTimeReliableMessageSentListener, OnInvitationReceivedListener, 
OnInvitationsLoadedListener, OnPlayersLoadedListener {
	private RoomConfig me;
	private RoomConfig.Builder myBuilder;
	private Bundle myMatchCriteria;
	private BA mBA;
	private String TAG = "RoomConfigWrapper";
	private IOnActivityResult ion;
	private String eventName;
	public static final int RESULT_LEFT_ROOM = GamesActivityResultCodes.RESULT_LEFT_ROOM;
	public static final int RESULT_OK = Activity.RESULT_OK;
	public static final int RESULT_CANCELED = Activity.RESULT_CANCELED;

	
	@Hide
	public RoomConfigWrapper(BA ba){
		myBuilder = null;
		me = null;
		mBA = ba;
	}
	public RoomConfigWrapper(){
		myBuilder = null;
		me = null;
		mBA = null;
	}
	
	/**
	 * used to set listeners and start the RoomConfig build process
	 * cannot be initialized until OnSignInSucceeded is called
	 */
	public void initialize(BA ba, String EventName, GameClientWrapper GCW){
		debugLog("entering RCW initialize");
		mBA = ba;
		myBuilder = RoomConfig.builder(this);
		myBuilder.setMessageReceivedListener(this);
		myBuilder.setRoomStatusUpdateListener(this);
		me = null;
		if(GCW.me != null){
			debugLog("inviteListenerRegistered");
			GCW.me.registerInvitationListener(this);
		}
		
		debugLog("initialized");
		eventName = EventName.toLowerCase(ba.cul);
		debugLog(eventName + " is eventName");
	}
	
 
	
	/**
	 * 
	 * returns an invitationID if one exists, null otherwise
	 */
	 public String getInvitationId(GameService gs) {
		 	return gs.mHelper.getInvitationId();
	    }

	/**
	 * sets the variant for the room when calling createRoom
	 */
	public void setVariant(int variant) {
		if(myBuilder!=null){
			myBuilder.setVariant(variant);
		}
	}
	
	/**
	 * Creates an auto-match criteria for anew invitation.
	 * minAutoMatchPlayers 	min number of auto-matched players.
	 * maxAutoMatchPlayers 	max number of auto-matched players.
	 * exclusiveBitMask 	exclusive bitmasks for the automatching request. 
	 * The logical AND of each pairing of automatching requests must equal zero for auto-match. 
	 * If there are no exclusivity requirements for the game, this value should just be set to 0.
	 */
	public void setMatchCriteria(int minAutoMatchPlayers, int maxAutoMatchPlayers, long exclusiveBitMask){
			myMatchCriteria = RoomConfig.createAutoMatchCriteria(minAutoMatchPlayers, maxAutoMatchPlayers, exclusiveBitMask);
	}
	
	
	
	/**
	 * sets whether to enable sockets for sending and receiving data
	 */
	public void setSocketCommunicationEnabled(boolean enableSockets) {
		if(myBuilder!=null){
			myBuilder.setSocketCommunicationEnabled(enableSockets);
		}
	}
	

	/**
	 * requires the _OnInvitePlayersComplete listener.
	 * Will leave current Activity, and start up new Activity that allows the selection of players to invite.
	 * returns list of invitees used in the Room Creation Process
	 */
	public void invitePlayers(final BA ba, final GameClientWrapper gameClient){
		if (gameClient.me.isConnected()) {
			Intent i = new Intent(gameClient.me.getSelectPlayersIntent(1, 3));
			
			ion = new IOnActivityResult() {
		        @Override
		        public void ResultArrived(int resultCode, Intent data) {
		            if (resultCode == Activity.RESULT_OK) {	
		            	// we got the result from the "select players" UI -- ready to create the room
		                handleResult(resultCode, data, ba, gameClient);	
		            } else {
		            	 List list1 = null;
		            	 ba.raiseEvent(this, eventName +"_oninviteplayerscomplete", list1);
		            }
		        }
		    };
		    ba.startActivityForResult(ion, i);
		}
		
	}
	

	
	private void handleResult(int resultCode, Intent data, BA ba, final GameClientWrapper gameClient) {
		debugLog("Select players UI succeeded.");

        final ArrayList<String> invitees = data.getStringArrayListExtra(GamesClient.EXTRA_PLAYERS);
        debugLog("Invitee count: " + invitees.size());
        Bundle autoMatchCriteria = null;
        int minAutoMatchPlayers = data.getIntExtra(GamesClient.EXTRA_MIN_AUTOMATCH_PLAYERS, 0);
        int maxAutoMatchPlayers = data.getIntExtra(GamesClient.EXTRA_MAX_AUTOMATCH_PLAYERS, 0);
        if (minAutoMatchPlayers > 0 || maxAutoMatchPlayers > 0) {
            autoMatchCriteria = RoomConfig.createAutoMatchCriteria(
                    minAutoMatchPlayers, maxAutoMatchPlayers, 0);
            debugLog("Automatch criteria: " + autoMatchCriteria);
            
        }
        
        ba.raiseEvent(this, eventName +"_oninviteplayerscomplete", invitees);
	}

	
	/**
	 * AddplayerId to invite to the room
	 */
	public void addPlayerToInvite(String playerId){
		myBuilder.addPlayersToInvite(playerId);
	}
	
	public void createRoom(GameClientWrapper gameClient){
		if(myBuilder!=null){
			 if (myMatchCriteria != null) {
				 	debugLog("used Auto match criteria");
		            myBuilder.setAutoMatchCriteria(myMatchCriteria);
		        }
			 gameClient.me.createRoom(myBuilder.build());
			 myMatchCriteria = null;
		}
	}
	
	/**
	 * add the Object returned by OnInvitePlayersComplete
	 */
	public void addPlayersToInvite(ArrayList<String> playerIds){
		debugLog("added player String");
		myBuilder.addPlayersToInvite(playerIds);
	}
	
	/**
	 * Shows a waitingRoom that will track user join progress
	 * response will be sent to _OnWaitingRoomResponse
	 */
    public void showWaitingRoom(RoomWrapper rw, GameClientWrapper gameClient, int NumberOfPlayers) {

        // minimum number of players required for our game
        final int MIN_PLAYERS = NumberOfPlayers;
        Intent i = gameClient.me.getRealTimeWaitingRoomIntent(rw.getRoom(), MIN_PLAYERS);
        ion = new IOnActivityResult() {
	        @Override
	        public void ResultArrived(int resultCode, Intent data) {
	        	debugLog("Waiting Room Done");
	            mBA.raiseEvent(this, eventName +"_onwaitingroomresponse", resultCode, data);
	        }
	    };
	    mBA.startActivityForResult(ion, i);
    }
	
    /**
     * will start up the room that will show invitations, if any
     * invitations will be delivered to OnInvitationsComplete listener
     */
    public void seeInvitations(GameClientWrapper gameClient){
    	Intent i = gameClient.me.getInvitationInboxIntent();
    	 ion = new IOnActivityResult() {
 	        @Override
 	        public void ResultArrived(int resultCode, Intent data) {
 	        	InvitationWrapper iw = new InvitationWrapper();
 	        	if (resultCode == Activity.RESULT_OK){
	 	            debugLog("Invitation inbox UI succeeded.");
	 	            Invitation inv = data.getExtras().getParcelable(GamesClient.EXTRA_INVITATION);
	 	            iw.me = inv;
 	        	} else {
 	        		iw = null;
 	        	}
 	        	debugLog("Invitation done");
 	            mBA.raiseEvent(this, eventName +"_oninvitationscomplete", resultCode, iw);
 	        }
 	    };
 	    mBA.startActivityForResult(ion, i);
    }
    
    
    /**
     * starts the joinRoom process with the associated inviteID
     */    
    public void joinRoom(String invitationID, GameClientWrapper gameClient){
    	if (myBuilder == null){
    		return;
    	}
    	myBuilder.setInvitationIdToAccept(invitationID);
    	gameClient.me.joinRoom(myBuilder.build());	
    }
    
    /**
     * Send a message to a participant in a real-time room reliably. 
     * The caller will receive a callback to report the status of the send message operation. 
     * Throws an IllegalArgumentException if recipientParticipantId is not a valid participant or belongs to the current player. 
     * The maximum message size supported is MAX_RELIABLE_MESSAGE_LEN bytes.
     * Store the int Token returned for use with OnRealTimeMessageSent Listener
     */
    public int sendReliableRealTimeMessage(RoomWrapper room, GameClientWrapper gameClient, String recipientParticipantId, byte[] messageData){
    	try {
    		return gameClient.me.sendReliableRealTimeMessage(this, messageData, room.getRoom().getRoomId(), recipientParticipantId);
		} catch (Exception e) {
			debugLog(e.toString());
		}	
    	return GamesClient.STATUS_REAL_TIME_MESSAGE_SEND_FAILED;
    }
    
    /**
     * Send a message to a participant in a real-time room unreliably. 
     * Throws an IllegalArgumentException if recipientParticipantId is not a valid participant or belongs to the current player. 
     * The maximum message size supported is MAX_UNRELIABLE_MESSAGE_LEN bytes.
     * returns STATUS_OK on a successful attempt, STATUS_REAL_TIME_MESSAGE_FAILED if the message failed to send. 
     */
    public int sendUnreliableRealTimeMessage(RoomWrapper room, GameClientWrapper gameClient, String recipientId, byte[] messageData){
    	try {
    		return gameClient.me.sendUnreliableRealTimeMessage(messageData, room.getRoom().getRoomId(), recipientId);
		} catch (Exception e) {
			debugLog(e.toString());
		}
    	return GamesClient.STATUS_REAL_TIME_MESSAGE_SEND_FAILED;	
    	
    	
    }
    
    /**
     * Called to notify the client that a reliable message was sent for a room.
     */
    @Hide
    @Override
	public void onRealTimeMessageSent(int statusCode, int tokenId,
			String recipientParticipantId) {
			mBA.raiseEvent(this, eventName + "_onrealtimemessagesent", statusCode, tokenId, recipientParticipantId);
		
	}
    
    /**
     * will activate when an invitation arrives
     */
    @Hide
    @Override
    public void onInvitationReceived(Invitation invitation){
    	debugLog("invite received");
    	Toast.makeText(mBA.context, "Got an invite", Toast.LENGTH_SHORT).show();
    	InvitationWrapper iw = new InvitationWrapper(mBA);
    	iw.me = invitation;
    	mBA.raiseEvent(this, eventName + "_oninvitationreceived", iw);
    }
    
    /**
     * Leave the specified room
     * This will disconnect the player from the room, but allow other players to continue playing the game. 
     * The result is delivered by the callback onLeftRoom listener if implemented
     */
    public void leaveRoom(GameClientWrapper gameClient, RoomWrapper rw){
    	if (rw.getRoom() == null) {
    		return;
    	}
    	gameClient.me.leaveRoom(this, rw.getRoom().getRoomId());
    }
    
    /**
     * Called when the client is connected to the connected set in a room
     */
	@Override
	public void onConnectedToRoom(Room room) {
		debugLog("onConnectedToRoom and # participants is " + Integer.toString(room.getParticipants().size()));
		RoomWrapper rw = new RoomWrapper(mBA, room);
		//mBA.raiseEvent(this, eventName + "_onconnectedtoroom", rw);
		mBA.raiseEventFromDifferentThread(this, null, 0,eventName + "_onconnectedtoroom" , false, new Object[] {rw});
		debugLog(eventName + "_onconnectedtoroom");
	}

	/**
	 * Called when the client is disconnected from the connected set in a room
	 */
	@Override
	public void onDisconnectedFromRoom(Room room) {
		Toast.makeText(mBA.context, "onDisconnectedFromRoom", Toast.LENGTH_SHORT).show();
		debugLog("onDisconnectedFromRoom");
	}

	/**
	 * Called when one or more peers decline the invitation to a room
	 */
	@Override
	public void onPeerDeclined(Room room, List<String> arg1) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		mBA.raiseEvent(this, eventName + "_onpeerdeclined", rw, list1);
		debugLog("onPeerDeclined");
	}

	/**
	 * Called when one or more peers are invited to a room
	 */
	@Override
	public void onPeerInvitedToRoom(Room room, List<String> arg1) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		mBA.raiseEvent(this, eventName + "_onpeerinvitedtoroom", rw, list1);
		debugLog("onPeerInvitedToRoom");
	}
	
	/**
	 * Called when one or more peer participants join a room
	 */
	@Override
	public void onPeerJoined(Room room, List<String> arg1) {
		debugLog("onPeerJoined");
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		mBA.raiseEvent(this, eventName + "_onpeerjoined", rw, list1);
	}
	
	/**
	 * Called when one or more peer participant leave a room
	 */
	@Override
	public void onPeerLeft(Room room, List<String> arg1) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		mBA.raiseEvent(this, eventName + "_onpeerleft", rw, list1);
		debugLog("onPeerLeft");
	}

	/**
	 * Called when one or more peer participants are connected to a room
	 */
	@Override
	public void onPeersConnected(Room room, List<String> arg1) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		//mBA.raiseEvent(this, eventName + "_onpeersconnected", rw, list1);
		mBA.raiseEventFromDifferentThread(this, null, 0,eventName + "_onpeersconnected" , false, new Object[] {rw, list1});
		debugLog("onPeersConnected");
	}

	/**
	 * Called when one or more peer participants are disconnected from a room
	 */
	@Override
	public void onPeersDisconnected(Room room, List<String> arg1) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		list1.Initialize();
		list1.getObject().addAll(arg1);
		mBA.raiseEvent(this, eventName + "_onpeersdisconnected", rw, list1);
		debugLog("onPeersDisconnected");
	}

	/**
	 * Called when the server has started the process of auto-matching
	 */
	@Override
	public void onRoomAutoMatching(Room room) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		mBA.raiseEvent(this, eventName + "_onroomautomatching", rw);
		debugLog("onRoomAutoMatching");
	}
	
	/**
	 * Called when one or more participants have joined the room,
	 *  and have started the process of establishing peer connection
	 */
	@Override
	public void onRoomConnecting(Room room) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		mBA.raiseEvent(this, eventName + "_onroomconnecting", rw);
		debugLog("onRoomConnecting");
	}

	/**
	 * Called when the client attempts to join a real-time room
	 */
	@Override
	public void onJoinedRoom(int statusCode, Room room) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		mBA.raiseEvent(this, eventName + "_onjoinedroom", statusCode, rw);
		debugLog("onJoinedRoom");
	}

	/**
	 * Called when the client attempts to leaves the real-time room
	 */
	@Override
	public void onLeftRoom(int statusCode, String roomId) {
		mBA.raiseEvent(this, eventName + "_onleftroom", statusCode, roomId);
		debugLog("onLeftRoom");
	}
	
	/**
	 * Called when all the participants in a real-time room are fully connected.
	 */
	@Override
	public void onRoomConnected(int statusCode, Room room) {
		RoomWrapper rw = new RoomWrapper(mBA, room);
		mBA.raiseEvent(this, eventName + "_onroomconnected", statusCode, rw);
		debugLog("onRoomConnected");
	}
	
	/**
	 * Called when the client attempts to create a real-time room
	 */
	@Override
	public void onRoomCreated(int statusCode, Room room) {
		mBA.raiseEvent(this, eventName + "_onroomcreated", statusCode, room);
		debugLog("onRoomCreated and statusCode: " + Integer.toString(statusCode));
	}

	/**
	 * Called to notify the client that a reliable or unreliable message was received for a room.
	 */
	@Override
	public void onRealTimeMessageReceived(RealTimeMessage message) {
		byte[]buf = message.getMessageData();
		boolean isReliable = message.isReliable();
		String particpantID = message.getSenderParticipantId();
		mBA.raiseEvent(this, eventName + "_onrealtimemessagerecieved", isReliable, buf, particpantID);
		//debugLog("onRealTimeMessageReceived");
	}
	
//	@Override
//	public void onConnected(Bundle connectionHint) {
//	    if (connectionHint != null) {
//	    	debugLog("bundle not null onConnected");
//	        Invitation inv =
//	            connectionHint.getParcelable(GamesClient.EXTRA_INVITATION);
//
//	
//	    } else {
//	    	debugLog("bundle null onConnected");
//	    }
//	}
	/**
	 * requires the OnInvitationsLoaded listener
	 */
	public void loadInvitations(GameClientWrapper gameClient) {
		//gameClient.eventName = eventName;
		gameClient.me.loadInvitations(this);
	}
	
	@Hide
	@Override
	public void onInvitationsLoaded(int arg0, InvitationBuffer arg1) {
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		if (arg1.getCount() > 0 ){
			list1.Initialize();
			for(int i=0; i<arg1.getCount(); i++){
				 Invitation inv = arg1.get(i);
				 InvitationWrapper iw = new InvitationWrapper();
				 iw.me = inv;
				 list1.Add(iw);
	        }
		} else {
			list1 = null;
		}
		mBA.raiseEvent(this, eventName + "_oninvitationsloaded", list1);
		debugLog("invitations are in.  There are " + Integer.toString(arg1.getCount()));
	}
	
	/**
	 * requires the OnPlayersLoaded listener
	 */
	public void loadInvitablePlayers(GameClientWrapper gameClient, int pageSize, boolean forceReload){
		gameClient.me.loadInvitablePlayers(this, pageSize, forceReload);
	}
	
	@Hide
	@Override
	public void onPlayersLoaded(int arg0, PlayerBuffer arg1) {
		anywheresoftware.b4a.objects.collections.List list1 = new anywheresoftware.b4a.objects.collections.List();
		if (arg1.getCount() > 0 ){
			list1.Initialize();
			for(int i=0; i<arg1.getCount(); i++){
				 Player p = arg1.get(i);
				 PlayerWrapper pw = new PlayerWrapper(p);
				 list1.Add(pw);
	        }
		} else {
			list1 = null;
		}
		mBA.raiseEvent(this, eventName + "_onplayersloaded", list1);
		debugLog("Players are in.  There are " + Integer.toString(arg1.getCount()));
		
	}
	
//	@Override
//	public void onDisconnected() {
//		// TODO Auto-generated method stub
//		
//	}

	
	private void debugLog(String message){
		Log.d(TAG, message);
	}

	

	

   
	
	
}
